/**
 * # NDDB: N-Dimensional Database
 * 
 * Copyright(c) 2012 Stefano Balietti
 * MIT Licensed
 * 
 * NDDB provides a simple, lightweight, NO-SQL object database 
 * for node.js and the browser. It depends on JSUS.
 * 
 * Allows to define any number of comparator and indexing functions, 
 * which are associated to any of the dimensions (i.e. properties) of 
 * the objects stored in the database. 
 * 
 * Whenever a comparison is needed, the corresponding comparator function 
 * is called, and the database is updated.
 * 
 * Whenever an object is inserted that matches one of the indexing functions
 * an hash is produced, and the element is added to one of the indexes.
 * 
 * Additional features are: methods chaining, tagging, and iteration 
 * through the entries.
 * 
 * 
 * See README.md for help.
 * 
 * ---
 * 
 */
(function(e,t,n){function u(e,n,r){e=e||{};if(!t)throw new Error("JSUS not found.");this.db=[],this.tags={},this.hooks={insert:[],remove:[]},this.nddb_pointer=0,u.compatibility.getter?this.__defineGetter__("length",function(){return this.db.length}):this.length=null,this.__C={},this.__H={},this.__I={},this.__update={},this.__update.pointer=!1,this.__update.indexes=!1,this.__update.sort=!1,this.__parent=r||undefined,this.init(e),this.importDB(n)}var r=null,i=[],s=function(e,t){return!e||!t?(u.log("Attempt to add invalid condition","ERR"),!1):(i.push({type:e,condition:t}),!0)},o=function(e,t,n,i){if(!r)return u.log("No operation found.","ERR"),!1;var o=this._analyzeQuery(t,n,i);return o?s(e,o):!1};u.prototype.and=u.prototype.AND=function(e,t,n){return o("AND",e,t,n)},u.prototype.or=u.prototype.OR=function(e,t,n){return o("OR",e,t,n)},u.prototype.not=u.prototype.NOT=function(e,t,n){return o("NOT",e,t,n)},u.compatibility=t.compatibility(),e.NDDB=u,u.log=console.log,u.__symbols=[">",">=",">==","<","<=","<==","!=","!==","=","==","===","><","<>","in","!in"],u.__operations=["select","groupby","limit","first","fetch","last"],u.decycle=function(e){return JSON&&JSON.decycle&&"function"==typeof JSON.decycle&&(e=JSON.decycle(e)),e},u.retrocycle=function(e){return JSON&&JSON.retrocycle&&"function"==typeof JSON.retrocycle&&(e=JSON.retrocycle(e)),e},u.prototype.init=function(e){e=e||{},this.__options=e,e.log&&(u.log=e.log),e.C&&(this.__C=e.C),e.H&&(this.__H=e.H),e.I&&(this.__I=e.I),e.tags&&(this.tags=e.tags),e.nddb_pointer>0&&(this.nddb_pointer=e.nddb_pointer),e.hooks&&(this.hooks=e.hook),e.update&&("undefined"!=typeof e.update.pointer&&(this.__update.pointer=e.update.pointer),"undefined"!=typeof e.update.indexes&&(this.__update.indexes=e.update.indexes),"undefined"!=typeof e.update.sort&&(this.__update.sort=e.update.sort))},u.prototype.globalCompare=function(e,t){return"undefined"==typeof e&&"undefined"==typeof t?0:"undefined"==typeof t?-1:"undefined"==typeof e?1:e.nddbid<t.nddbid?-1:e.nddbid>t.nddbid?1:0},u.prototype._masquerade=function(e,t){return"undefined"==typeof e?!1:"undefined"!=typeof e.nddbid?e:(t=t||this.db,u.compatibility.defineProperty?Object.defineProperty(e,"nddbid",{value:t.length,configurable:!0,writable:!0}):e.nddbid=t.length,e)},u.prototype._masqueradeDB=function(e){if(!e)return[];var t=[];for(var n=0;n<e.length;n++)t[n]=this._masquerade(e[n],t);return t},u.prototype._autoUpdate=function(e){var n=e?t.merge(e,this.__update):this.__update;n.pointer&&(this.nddb_pointer=this.db.length-1),n.sort&&this.sort(),n.indexes&&this.rebuildIndexes(),this.__parent&&this.__parent._autoUpdate(n)},u.prototype.importDB=function(e){if(!e)return;this.db||(this.db=[]);for(var t=0;t<e.length;t++)this.insert(e[t])},u.prototype.insert=function(e){if("undefined"==typeof e||e===null)return;this.db||(this.db=[]),this._insert(e)},u.prototype._insert=function(e){e=this._masquerade(e),this.db.push(e),this.emit("insert",e),this.__update.indexes&&(this._hashIt(e),this._indexIt(e)),this._autoUpdate({indexes:!1})},u.prototype.breed=function(e){e=e||this.db;var t=this.cloneSettings(),n=this.__parent||this;return new this.constructor(t,e,n)},u.prototype.cloneSettings=function(){var e=this.__options||{};return e.H=this.__H,e.I=this.__I,e.C=this.__C,e.tags=this.tags,e.update=this.__update,t.clone(e)},u.prototype.toString=function(){var e="";for(var t=0;t<this.db.length;t++)e+=this.db[t]+"\n";return e},u.prototype.stringify=function(e){if(!this.length)return"[]";e="undefined"==typeof e?!0:e;var n;e?n=function(e){return t.isEmpty(e)?"{}":JSON.stringify(e)}:n=function(e){return t.isEmpty(e)?"{}":JSON.stringify(e,null,4)};var r="[";return this.each(function(e){e=u.decycle(e),r+=n(e)+", "}),r=r.replace(/, $/,"]"),r},u.prototype.compare=u.prototype.c=function(e,t){return!e||!t?(u.log("Cannot set empty property or empty comparator","ERR"),!1):(this.__C[e]=t,!0)},u.prototype.comparator=function(e){return"undefined"!=typeof this.__C[e]?this.__C[e]:function(n,r){if("undefined"==typeof n&&"undefined"==typeof r)return 0;if("undefined"==typeof n)return 1;if("undefined"==typeof r)return-1;var i=t.getNestedValue(e,n),s=t.getNestedValue(e,r);return"undefined"==typeof i&&"undefined"==typeof s?0:"undefined"==typeof i?1:"undefined"==typeof s?-1:i>s?1:s>i?-1:0}},u.prototype.isReservedWord=function(e){return this[e]?!0:!1},u.prototype._isValidIndex=function(e){if("undefined"==typeof e)return u.log("A valid index name must be provided","ERR"),!1;if(this.isReservedWord(e)){var t="A reserved word have been selected as an index. ";return t+="Please select another one: "+e,u.log(t,"ERR"),!1}return!0},u.prototype.hash=u.prototype.h=function(e,t){return this._isValidIndex(e)?(this.__H[e]=t||Object.toString,this[e]={},!0):!1},u.prototype.index=u.prototype.i=function(e,t){return this._isValidIndex(e)?(this.__I[e]=t||Object.toString,this[e]={},!0):!1},u.prototype.rebuildIndexes=function(){var e=!1,n=!1;if(!t.isEmpty(this.__H)){e=!0;for(var r in this.__H)this.__H.hasOwnProperty(r)&&(this[r]={})}if(!t.isEmpty(this.__I)){n=!0;for(var r in this.__I)this.__I.hasOwnProperty(r)&&(this[r]={})}e&&!n?this.each(this._hashIt):!e&&n?this.each(this._indexIt):e&&n&&this.each(function(e){this.hashIt(e),this.indexIt(e)})},u.prototype._hashIt=function(e){if(!e)return!1;if(t.isEmpty(this.__H))return!1;var n=null,r=null,i=null;for(var s in this.__H)if(this.__H.hasOwnProperty(s)){n=this.__H[s],i=n(e);if("undefined"==typeof i)continue;this[s]||(this[s]={}),this[s][i]||(this[s][i]=new u),this[s][i].insert(e)}},u.prototype._indexIt=function(e){if(!e)return!1;if(t.isEmpty(this.__I))return!1;var n=null,r=null,i=null;for(var s in this.__I)if(this.__I.hasOwnProperty(s)){n=this.__I[s],i=n(e);if("undefined"==typeof i)continue;this[s]||(this[s]={}),this[s][i]=e}},u.prototype.on=function(e,t){if(!e||!t||!this.hooks[e])return;return this.hooks[e].push(t),!0},u.prototype.off=function(e,t){if(!e||!this.hooks[e]||!this.hooks[e].length)return;if(!t)return this.hooks[e]=[],!0;for(var n=0;n<this.hooks[e].length;n++)if(this.hooks[e][n]==t)return this.hooks[e].splice(n,1),!0;return!1},u.prototype.emit=function(e,t){if(!e||!this.hooks[e]||!this.hooks[e].length)return;for(var n=0;n<this.hooks[e].length;n++)this.hooks[e][n].call(this,t)},u.prototype._analyzeQuery=function(e,n,r){var i=function(e,t,n){var r="(?)",i="Malformed query: "+e||r+" "+t||r+" "+n||r;return u.log(i,"WARN"),!1};"undefined"==typeof e&&i(e,n,r);if("undefined"!=typeof n){"undefined"==typeof r&&i(e,n,r);if(!t.in_array(n,[">",">=",">==","<","<=","<==","!=","!==","=","==","===","><","<>","in","!in"]))return u.log("Query error. Invalid operator detected: "+n,"WARN"),!1;n==="="&&(n="==");if(t.in_array(n,["><","<>","in","!in"])){r instanceof Array||(u.log("Range-queries need an array as third parameter","WARN"),i(e,n,r));if(n==="<>"||n==="><")r[0]=t.setNestedValue(e,r[0]),r[1]=t.setNestedValue(e,r[1])}else r=t.setNestedValue(e,r)}else"undefined"!=typeof r?i(e,n,r):(n="",r="");return{d:e,op:n,value:r}},u.prototype.distinct=function(){return this.breed(t.distinct(this.db))},u.prototype.select=function(e,n,r){var i=this._analyzeQuery(e,n,r);if(!i)return!1;var e=i.d,n=i.op,r=i.value,s=this.comparator(e),o=function(n){if("undefined"!=typeof t.getNestedValue(e,n))return n},a=function(i){try{if(t.eval(s(i,r)+n+0,i))return i}catch(o){return u.log("Malformed select query: "+e+n+r),!1}},f=function(e){if(s(e,r[0])>0&&s(e,r[1])<0)return e},l=function(e){if(s(e,r[0])<0&&s(e,r[1]>0))return e},c=function(n){if(t.in_array(t.getNestedValue(e,n),r))return n},h=function(n){if(!t.in_array(t.getNestedValue(e,n),r))return n};switch(n){case"":var p=o;break;case"<>":var p=l;break;case"><":var p=f;break;case"in":var p=c;break;case"!in":var p=h;break;default:var p=a}return this.filter(p)},u.prototype.limit=function(e){e=e||0;if(e===0)return this.breed();var t=e>0?this.db.slice(0,e):this.db.slice(e);return this.breed(t)},u.prototype.reverse=function(){return this.db.reverse(),this},u.prototype.sort=function(e){if(!e)var t=this.globalCompare;else if("function"==typeof e)var t=e;else if(e instanceof Array)var n=this,t=function(t,r){for(var i=0;i<e.length;i++){var s=n.comparator(e[i]).call(n,t,r);if(s!==0)return s}return s};else var t=this.comparator(e);return this.db.sort(t),this},u.prototype.shuffle=function(){return this.db=t.shuffle(this.db),this},u.prototype.filter=function(e){return this.breed(this.db.filter(e))},u.prototype.each=u.prototype.forEach=function(){if(arguments.length===0)return;var e=arguments[0];for(var t=0;t<this.db.length;t++)arguments[0]=this.db[t],e.apply(this,arguments)},u.prototype.map=function(){if(arguments.length===0)return;var e=arguments[0],t=[],n=undefined;for(var r=0;r<this.db.length;r++)arguments[0]=this.db[r],n=e.apply(this,arguments),"undefined"!=typeof n&&t.push(n);return t},u.prototype.remove=function(){if(!this.length)return this;if(this.__parent){for(var e=0;e<this.db.length;e++){var t=this.db[e].nddbid-e;this.__parent.db.splice(t,1)}for(var e=0;e<this.__parent.length;e++)this.__parent.db[e].nddbid=e}return this.emit("remove",this.db),this.db=[],this._autoUpdate(),this},u.prototype.clear=function(e){return e?(this.db=[],this._autoUpdate()):u.log("Do you really want to clear the current dataset? Please use clear(true)","WARN"),e},u.prototype.join=function(e,n,r,i){return this._join(e,n,t.equals,r,i)},u.prototype.concat=function(e,t,n,r){return this._join(e,t,function(){return!0},n,r)},u.prototype._join=function(e,n,r,i,s){r=r||t.equals,i="undefined"!=typeof i?i:"joined";if(s)var s=s instanceof Array?s:[s];var o=[],a=[];for(var f=0;f<this.db.length;f++)try{var l=t.eval("this."+e,this.db[f]);if("undefined"!=typeof l)for(var c=f+1;c<this.db.length;c++)try{var h=t.eval("this."+n,this.db[c]);if("undefined"!=typeof h&&r(l,h)){var p=t.clone(this.db[f]),d=s?t.subobj(this.db[c],s):this.db[c];p[i]=d,o.push(p)}}catch(v){u.log("Key not found in entry: "+n,"WARN")}}catch(v){u.log("Key not found in entry: "+e,"WARN")}return this.breed(o)},u.prototype.split=function(e){var n=[];for(var r=0;r<this.db.length;r++)n=n.concat(t.split(this.db[r],e));return this.breed(n)},u.prototype._fetch=function(e,n){function r(e,n){return t.getNestedValue(n,e)}function i(e,n){var r=t.getNestedValue(n,e);if("undefined"!=typeof r)return t.obj2KeyedArray(r)}function s(e,n){var r=t.getNestedValue(n,e);if("undefined"!=typeof r)return n.split(".").concat(t.obj2KeyedArray(r))}switch(n){case"VALUES":var o=e?i:t.obj2Array;break;case"KEY_VALUES":var o=e?s:t.obj2KeyedArray;break;default:if(!e)return this.db;var o=r}var u=[];for(var a=0;a<this.db.length;a++){var f=o.call(this.db[a],this.db[a],e);"undefined"!=typeof f&&u.push(f)}return u},u.prototype.fetch=function(e){return this._fetch(e,!0)},u.prototype.fetchArray=function(e){return this._fetch(e,"VALUES")},u.prototype.fetchKeyArray=function(e){return this._fetch(e,"KEY_VALUES")},u.prototype.fetchValues=function(e){return this._fetch(e,"VALUES")},u.prototype.fetchKeyValues=function(e){return this._fetch(e,"KEY_VALUES")},u.prototype.groupBy=function(e){if(!e)return this.db;var n=[],r=[];for(var i=0;i<this.db.length;i++){var s=t.getNestedValue(e,this.db[i]);if("undefined"==typeof s)continue;if(!t.in_array(s,n)){n.push(s);var o=this.filter(function(n){if(t.equals(t.getNestedValue(e,n),s))return this});o.nddb_pointer=0,r.push(o)}}return r},u.prototype.count=function(e){if("undefined"==typeof e)return this.db.length;var n=0;for(var r=0;r<this.db.length;r++)t.hasOwnNestedProperty(e,this.db[r])&&n++;return n},u.prototype.sum=function(e){if("undefined"==typeof e)return!1;var n=0;for(var r=0;r<this.db.length;r++){var i=t.getNestedValue(e,this.db[r]);isNaN(i)||(n+=i)}return n},u.prototype.mean=function(e){if("undefined"==typeof e)return!1;var n=0,r=0;for(var i=0;i<this.db.length;i++){var s=t.getNestedValue(e,this.db[i]);isNaN(s)||(n+=s,r++)}return r===0?0:n/r},u.prototype.stddev=function(e){if("undefined"==typeof e)return!1;var n=this.mean(e);if(isNaN(n))return!1;var r=0;return this.each(function(i){var s=t.getNestedValue(e,i);isNaN(s)||(r+=Math.pow(s-n,2))}),r!==0?Math.sqrt(r):0},u.prototype.min=function(e){if("undefined"==typeof e)return!1;var n=!1;for(var r=0;r<this.db.length;r++){var i=t.getNestedValue(e,this.db[r]);!isNaN(i)&&(i<n||n===!1)&&(n=i)}return n},u.prototype.max=function(e){if("undefined"==typeof e)return!1;var n=!1;for(var r=0;r<this.db.length;r++){var i=t.getNestedValue(e,this.db[r]);!isNaN(i)&&(i>n||n===!1)&&(n=i)}return n},u.prototype.skim=function(e){return e?this.breed(this.map(function(n){var r=t.skim(n,e);if(!t.isEmpty(r))return r})):this},u.prototype.keep=function(e){return e?this.breed(this.map(function(n){var r=t.subobj(n,e);if(!t.isEmpty(r))return r})):this.breed([])},u.prototype.diff=function(e){return!e||!e.length?this:("object"==typeof e&&(e instanceof u||e instanceof this.constructor)&&(e=e.db),this.breed(t.arrayDiff(this.db,e)))},u.prototype.intersect=function(e){if(!e||!e.length)return this;if("object"==typeof e)if(e instanceof u||e instanceof this.constructor)var e=e.db;return this.breed(t.arrayIntersect(this.db,e))},u.prototype.get=function(e){var e=e||this.nddb_pointer;return e<0||e>this.db.length-1?!1:this.db[e]},u.prototype.next=function(){var e=u.prototype.get.call(this,++this.nddb_pointer);return e||this.nddb_pointer--,e},u.prototype.previous=function(){var e=u.prototype.get.call(this,--this.nddb_pointer);return e||this.nddb_pointer++,e},u.prototype.first=function(e){var t=this.fetch(e);return t.length?(this.nddb_pointer=t[0].nddbid,t[0]):undefined},u.prototype.last=function(e){var t=this.fetch(e);return t.length?(this.nddb_pointer=t[t.length-1].nddbid,t[t.length-1]):undefined},u.prototype.tag=function(e,t){if("undefined"==typeof e)return u.log("Cannot register empty tag.","ERR"),!1;var n=null,r=typeof t;if(r==="undefined")n=this.db[this.nddb_pointer];else if(r==="number"){if(t>this.length||t<0)return u.log("Invalid index provided for tag registration","ERR"),!1;n=this.db[t]}else n=t;return this.tags[e]=n,!0},u.prototype.resolveTag=function(e){return"undefined"==typeof e?(u.log("Cannot resolve empty tag.","ERR"),!1):this.tags[e]};var a=function(){return"function"==typeof n};if(t.isNodeJS()){require("./external/cycle.js");var f=require("fs")}u.prototype.save=function(e,r,i){if(!e)return u.log("You must specify a valid file / id.","ERR"),!1;i=i||!1;if(!t.isNodeJS())return a()?(n(e,this.stringify(i)),r&&r(),!0):(u.log("No support for persistent storage found.","ERR"),!1);f.writeFile(e,this.stringify(i),"utf-8",function(e){if(e)throw e;return r&&r(),!0})},u.prototype.load=function(e,r){if(!e)return u.log("You must specify a valid file / id.","ERR"),!1;if(!t.isNodeJS()){if(!a())return u.log("No support for persistent storage found.","ERR"),!1;var i=n(e);return this.importDB(i),r&&r(),!0}var s=function(e){var t=JSON.parse(e.toString()),n;for(n=0;n<t.length;n++)t[n]=u.retrocycle(t[n]);this.importDB(t)};if(!r){var o=f.readFileSync(e,"utf-8");s.call(this,o)}else f.readFile(e,"utf-8",function(e,t){if(e)throw e;s.call(this,t),r()})}})("undefined"!=typeof module&&"undefined"!=typeof module.exports?module.exports:window,"undefined"!=typeof JSUS?JSUS:module.parent.exports.JSUS||require("JSUS").JSUS,"object"==typeof module&&"function"==typeof require?module.parent.exports.store||require("shelf.js/build/shelf-fs.js").store:this.store)