/**
 * 
 * # GameWindow
 * 
 * Copyright(c) 2012 Stefano Balietti
 * MIT Licensed
 * 
 * GameWindow provides a handy API to interface nodeGame with the 
 * browser window.
 * 
 * Creates a custom root element inside the HTML page, and insert an
 * iframe element inside it.
 * 
 * Dynamic content can be loaded inside the iframe without losing the
 * javascript state inside the page.
 * 
 * Defines a number of pre-defined profiles associated with special
 * configuration of widgets.
 * 
 * Depends on nodegame-client. 
 * GameWindow.Table and GameWindow.List depend on NDDB and JSUS.
 * 
 * Widgets can have custom dependencies, which are checked internally 
 * by the GameWindow engine.
 * 
 * 
 */
(function(e,t){function f(){var n=this;if("undefined"==typeof e)throw new Error("nodeWindow: no DOM found. Are we in a browser? Aborting.");"undefined"==typeof t&&t.log("nodeWindow: nodeGame not found","ERR"),t.log("nodeWindow: loading..."),this.frame=null,this.mainframe="mainframe",this.root=null,this.conf={},this.state=s.iss.LOADED,this.areLoading=0,this.init()}var n=t.JSUS,r=t.Player,i=t.PlayerList,s=t.GameState,o=t.GameMsg,u=t.GameMsgGenerator,a=n.get("DOM");if(!a)throw new Error("DOM object not found. Aborting");f.prototype=a,f.prototype.constructor=f,f.defaults={},f.defaults.promptOnleave=!0,f.defaults.noEscape=!0,f.prototype.init=function(e){e=e||{},this.conf=n.merge(f.defaults,e),this.conf.promptOnleave?this.promptOnleave():this.conf.promptOnleave===!1&&this.restoreOnleave(),this.conf.noEscape?this.noEscape():this.conf.noEscape===!1&&this.restoreEscape()},f.prototype.getElementById=function(e){var t=null;return this.frame&&this.frame.getElementById&&(t=this.frame.getElementById(e)),t||(t=document.getElementById(e)),t},f.prototype.getElementsByTagName=function(e){return this.frame?this.frame.getElementsByTagName(e):document.getElementsByTagName(e)},f.prototype.setup=function(n){this.root||(this.root=document.body);switch(n){case"MONITOR":t.widgets.append("NextPreviousState"),t.widgets.append("GameSummary"),t.widgets.append("StateDisplay"),t.widgets.append("StateBar"),t.widgets.append("DataBar"),t.widgets.append("MsgBar"),t.widgets.append("GameBoard"),t.widgets.append("ServerInfoDisplay"),t.widgets.append("Wall"),t.conf.host&&this.addCSS(document.body,t.conf.host+"/stylesheets/monitor.css");break;case"PLAYER":this.header=this.generateHeader();var r=this.addIFrame(this.root,"mainframe");t.game.vs=t.widgets.append("VisualState",this.header),t.game.timer=t.widgets.append("VisualTimer",this.header),t.game.sd=t.widgets.append("StateDisplay",this.header),t.widgets.append("WaitScreen"),t.conf.host&&this.addCSS(document.body,t.conf.host+"/stylesheets/player.css"),this.frame=e.frames[this.mainframe];var i=this.getBlankPage();this.conf.noEscape,e.frames[this.mainframe].src=i}},f.prototype.load=f.prototype.loadFrame=function(n,r,i){if(!n)return;i=i||this.mainframe,this.state=s.iss.LOADING,this.areLoading++;var o=this,u=document.getElementById(i);u.onload=function(){o.conf.noEscape,o.updateStatus(r,i)},e.frames[i].location=n,e.frames[i].window.node=t},f.prototype.updateStatus=function(n,r){this.frame=e.frames[r].document,n&&n.call(t.game),this.areLoading--,this.areLoading===0?(this.state=s.iss.LOADED,t.emit("WINDOW_LOADED")):t.log("Attempt to update state, before the window object was loaded","DEBUG")},f.prototype.generateHeader=function(){return this.header&&(this.header.innerHTML="",this.header=null),this.addElement("div",this.root,"gn_header")},f.prototype._write=a.write,f.prototype._writeln=a.writeln,f.prototype.write=function(e,n){var n=n||this.getScreen();return n?this._write(n,e):(t.log("Could not determine where writing","ERR"),!1)},f.prototype.writeln=function(e,n,r){var n=n||this.getScreen();return n?this._writeln(n,e,r):(t.log("Could not determine where writing","ERR"),!1)},f.prototype.toggleInputs=function(e,t){if("undefined"!=typeof e)var n=this.getElementById(e);if("undefined"==typeof n)var n=this.frame.body;var r=["button","select","textarea","input"],i=0;for(;i<r.length;i++){var s=n.getElementsByTagName(r[i]),o=0,u=s.length;for(;o<u;o++)state="undefined"!=typeof t?t:s[o].disabled?!1:!0,state?s[o].disabled=state:s[o].removeAttribute("disabled")}},f.prototype._generateRoot=function(e,t){var e=e||document.body||document.lastElementChild;return e||(this.addElement("body",document),e=document.body),this.root=this.addElement("div",e,t),this.root},f.prototype.generateNodeGameRoot=function(e){return this._generateRoot(e,"nodegame")},f.prototype.generateRandomRoot=function(e,t){return this._generateRoot(e,this.generateUniqueId())},f.prototype.getEventButton=function(e,n,r,i){if(!e)return;var s=this.getButton(r,n,i);return s.onclick=function(){t.emit(e)},s},f.prototype.addEventButton=function(e,t,n,r,i){if(!e)return;if(!n)var n=this.getScreen();var s=this.getEventButton(e,t,r,i);return n.appendChild(s)},f.prototype.getRecipientSelector=function(e){var t=document.createElement("select");return"undefined"!=typeof e&&(t.id=e),this.addStandardRecipients(t),t},f.prototype.addRecipientSelector=function(e,t){if(!e)return!1;var n=this.getRecipientSelector(t);return e.appendChild(n)},f.prototype.addStandardRecipients=function(e){var t=document.createElement("option");t.value="ALL",t.appendChild(document.createTextNode("ALL")),e.appendChild(t);var t=document.createElement("option");t.value="SERVER",t.appendChild(document.createTextNode("SERVER")),e.appendChild(t)},f.prototype.populateRecipientSelector=function(e,t){if("object"!=typeof t||"object"!=typeof e)return;this.removeChildrenFromNode(e),this.addStandardRecipients(e);var r,i;r=t.db||t,n.each(r,function(t){i=document.createElement("option"),i.value=t.id,i.appendChild(document.createTextNode(t.name||t.id)),e.appendChild(i)})},f.prototype.getActionSelector=function(e){var n=document.createElement("select");return"undefined"!=typeof e&&(n.id=e),this.populateSelect(n,t.actions),n},f.prototype.addActionSelector=function(e,t){if(!e)return;var n=this.getActionSelector(t);return e.appendChild(n)},f.prototype.getTargetSelector=function(e){var n=document.createElement("select");return"undefined"!=typeof e&&(n.id=e),this.populateSelect(n,t.targets),n},f.prototype.addTargetSelector=function(e,t){if(!e)return;var n=this.getTargetSelector(t);return e.appendChild(n)},f.prototype.getStateSelector=function(e){var t=this.getTextInput(e);return t},f.prototype.addStateSelector=function(e,t){if(!e)return;var n=this.getStateSelector(t);return e.appendChild(n)},f.prototype.generateUniqueId=function(e){var t=""+(e||n.randomInt(0,1e3)),r=this.getElementById(t);while(r)t=""+e+"_"+n.randomInt(0,1e3),r=this.getElementById(t);return t},f.prototype.noEscape=function(t){t=t||e,t.document.onkeydown=function(t){var n=e.event?event.keyCode:t.keyCode;if(n===27)return!1}},f.prototype.restoreEscape=function(t){t=t||e,t.document.onkeydown=null},f.prototype.promptOnleave=function(t,n){t=t||e,n="undefined"==typeof n?this.conf.textOnleave:n,t.onbeforeunload=function(t){return t=t||e.event,t&&(t.returnValue=n),n}},f.prototype.restoreOnleave=function(t){t=t||e,t.onbeforeunload=null},f.prototype.getScreen=function(){var e=this.frame;return e?e=this.frame.body||e:e=document.body||document.lastElementChild,e},f.prototype.getFrame=function(){return this.frame=e.frames.mainframe.document},t.window=new f,"undefined"!=typeof e&&(e.W=t.window)})("undefined"!=typeof window?window:module.parent.exports.window,"undefined"!=typeof window?window.node:module.parent.exports.node),function(e,t){if(!e)return console.log("nodegame-window: node object not found."),!1;if(!t)return e.err("window object not found.","nodegame-window"),!1;e.on("NODEGAME_GAME_CREATED",function(){t.init(e.conf.window)}),e.on("HIDE",function(n){var r=t.getElementById(n);if(!r){e.log("Cannot hide element "+n);return}r.style.visibility="hidden"}),e.on("SHOW",function(n){var r=t.getElementById(n);if(!r){e.log("Cannot show element "+n);return}r.style.visibility="visible"}),e.on("TOGGLE",function(n){var r=t.getElementById(n);if(!r){e.log("Cannot toggle element "+n);return}r.style.visibility==="visible"?r.style.visibility="hidden":r.style.visibility="visible"}),e.on("INPUT_DISABLE",function(e){t.toggleInputs(e,!0)}),e.on("INPUT_ENABLE",function(e){t.toggleInputs(e,!1)}),e.on("INPUT_TOGGLE",function(e){t.toggleInputs(e)}),e.log("node-window: listeners added")}("undefined"!=typeof node?node:undefined,"undefined"!=typeof node.window?node.window:undefined),function(e){function t(e){this.canvas=e,this.ctx=e.getContext("2d"),this.centerX=e.width/2,this.centerY=e.height/2,this.width=e.width,this.height=e.height}e.Canvas=t,t.prototype={constructor:t,drawOval:function(e){var t=e.x/e.scale_x,n=e.y/e.scale_y,r=e.radius||100;this.ctx.lineWidth=e.lineWidth||1,this.ctx.strokeStyle=e.color||"#000000",this.ctx.save(),this.ctx.scale(e.scale_x,e.scale_y),this.ctx.beginPath(),this.ctx.arc(t,n,r,0,Math.PI*2,!1),this.ctx.stroke(),this.ctx.closePath(),this.ctx.restore()},drawLine:function(e){var t=e.x,n=e.y,r=e.length,i=e.angle,s=-Math.cos(i)*r+e.x,o=Math.sin(i)*r+e.y;this.ctx.lineWidth=e.lineWidth||1,this.ctx.strokeStyle=e.color||"#000000",this.ctx.save(),this.ctx.beginPath(),this.ctx.moveTo(t,n),this.ctx.lineTo(s,o),this.ctx.stroke(),this.ctx.closePath(),this.ctx.restore()},scale:function(e,t){this.ctx.scale(e,t),this.centerX=this.canvas.width/2/e,this.centerY=this.canvas.height/2/t},clear:function(){this.ctx.clearRect(0,0,this.width,this.height);var e=this.canvas.width;this.canvas.width=1,this.canvas.width=e}}}(node.window),function(e,t,n){function o(e){this.options=e||{},this.tm=new s,this.init(this.options)}function u(e){e=e||{},this.content="undefined"!=typeof e.content?e.content:"",this.className="undefined"!=typeof e.style?e.style:null}var r=t.document,i=n.JSUS,s=n.TriggerManager;e.HTMLRenderer=o,e.HTMLRenderer.Entity=u,o.prototype.init=function(e){e=e||this.options,this.options=e,this.reset(),e.returnAt&&(this.tm.returnAt=e.returnAt),e.pipeline&&this.tm.initTriggers(e.pipeline)},o.prototype.reset=function(){this.clear(!0),this.addDefaultPipeline()},o.prototype.addDefaultPipeline=function(){this.tm.addTrigger(function(e){return r.createTextNode(e.content)}),this.tm.addTrigger(function(e){if(!e)return;if(e.content&&"object"==typeof e.content){var t=r.createElement("div");for(var n in e.content)if(e.content.hasOwnProperty(n)){var i=n+":	"+e.content[n];t.appendChild(r.createTextNode(i)),t.appendChild(r.createElement("br"))}return t}}),this.tm.addTrigger(function(e){if(!e)return;if(e.content&&e.content.parse&&"function"==typeof e.content.parse){var t=e.content.parse();if(i.isElement(t)||i.isNode(t))return t}}),this.tm.addTrigger(function(e){if(!e)return;if(i.isElement(e.content)||i.isNode(e.content))return e.content})},o.prototype.clear=function(e){return this.tm.clear(e)},o.prototype.addRenderer=function(e,t){return this.tm.addTrigger(e,t)},o.prototype.removeRenderer=function(e){return this.tm.removeTrigger(e)},o.prototype.render=function(e){return this.tm.pullTriggers(e)},o.prototype.size=function(){return this.tm.triggers.length}}("undefined"!=typeof node?node.window||node:module.exports,"undefined"!=typeof window?window:module.parent.exports.window,"undefined"!=typeof node?node:module.parent.exports.node),function(e,t){function o(e,t){e=e||{},this.options=e,r.call(this,e,t),this.id=e.id||"list_"+Math.round(Math.random()*1e3),this.DL=null,this.auto_update=this.options.auto_update||!1,this.htmlRenderer=null,this.lifo=!1,this.init(this.options)}function u(e){s.call(this,e),this.dt="undefined"!=typeof e.dt?e.dt:null,"undefined"!=typeof e.dd&&(this.dd=e.dd)}var n=t.JSUS,r=t.NDDB,i=t.window.HTMLRenderer,s=t.window.HTMLRenderer.Entity;e.List=o,o.prototype=new r,o.prototype.constructor=o,o.prototype.init=function(e){e=e||this.options,this.FIRST_LEVEL=e.first_level||"dl",this.SECOND_LEVEL=e.second_level||"dt",this.THIRD_LEVEL=e.third_level||"dd",this.last_dt=0,this.last_dd=0,this.auto_update="undefined"!=typeof e.auto_update?e.auto_update:this.auto_update;var t=this.lifo="undefined"!=typeof e.lifo?e.lifo:this.lifo;this.globalCompare=function(e,n){if(!e&&!n)return 0;if(!n)return 1;if(!e)return-1;if(!t){if(e.dt<n.dt)return-1;if(e.dt>n.dt)return 1}else{if(e.dt<n.dt)return 1;if(e.dt>n.dt)return-1}if(e.dt===n.dt){if("undefined"==typeof e.dd)return-1;if("undefined"==typeof n.dd)return 1;if(e.dd<n.dd)return-1;if(e.dd>n.dd)return 1;if(e.nddbid<n.nddbid)return 1;if(e.nddbid>n.nddbid)return-1}return 0},this.DL=e.list||document.createElement(this.FIRST_LEVEL),this.DL.id=e.id||this.id,e.className&&(this.DL.className=e.className),this.options.title&&this.DL.appendChild(document.createTextNode(e.title)),this.htmlRenderer=new i({render:e.render})},o.prototype._add=function(e){if(!e)return;this.insert(e),this.auto_update&&this.parse()},o.prototype.addDT=function(e,t){if("undefined"==typeof e)return;this.last_dt++,t="undefined"!=typeof t?t:this.last_dt,this.last_dd=0;var n=new u({dt:t,content:e});return this._add(n)},o.prototype.addDD=function(e,t,n){if("undefined"==typeof e)return;t="undefined"!=typeof t?t:this.last_dt,n="undefined"!=typeof n?n:this.last_dd++;var r=new u({dt:t,dd:n,content:e});return this._add(r)},o.prototype.parse=function(){this.sort();var e=null,t=null,n=function(){var n=document.createElement(this.SECOND_LEVEL);return this.DL.appendChild(n),t=null,e=n,n},r=function(){var e=document.createElement(this.THIRD_LEVEL);return this.DL.appendChild(e),e};if(this.DL){while(this.DL.hasChildNodes())this.DL.removeChild(this.DL.firstChild);this.options.title&&this.DL.appendChild(document.createTextNode(this.options.title))}for(var i=0;i<this.db.length;i++){var s=this.db[i],o;"undefined"==typeof s.dd?o=n.call(this):o=r.call(this);var u=this.htmlRenderer.render(s);o.appendChild(u)}return this.DL},o.prototype.getRoot=function(){return this.DL},u.prototype=new s,u.prototype.constructor=u}("undefined"!=typeof node?"undefined"!=typeof node.window?node.window:node:module.parent.exports,"undefined"!=typeof node?node:module.parent.exports),function(e,t,n){function a(e,t,n){e=e||{},a.log=e.log||a.log,this.defaultDim1=e.defaultDim1||"x",this.defaultDim2=e.defaultDim2||"y",this.defaultDim3=e.defaultDim3||"z",this.table=e.table||r.createElement("table"),this.id=e.id||"table_"+Math.round(Math.random()*1e3),this.auto_update="undefined"!=typeof e.auto_update?e.auto_update:!1,this.missing=e.missing||"missing",this.pointers={x:e.pointerX||0,y:e.pointerY||0,z:e.pointerZ||0},this.header=[],this.footer=[],this.left=[],this.right=[],s.call(this,e,t,n),this.options=this.__options}function f(e){u.call(this,e),this.x="undefined"!=typeof e.x?e.x:null,this.y="undefined"!=typeof e.y?e.y:null,this.z="undefined"!=typeof e.z?e.z:null}var r=t.document;e.Table=a,e.Table.Cell=f;var i=n.JSUS,s=n.NDDB,o=n.window.HTMLRenderer,u=n.window.HTMLRenderer.Entity;a.prototype=i.clone(s.prototype),a.prototype.constructor=a,a.H=["x","y","z"],a.V=["y","x","z"],a.log=n.log,a.prototype.init=function(e){s.prototype.init.call(this,e),e=e||this.options,"undefined"!=typeof e.id&&(this.table.id=e.id,this.id=e.id),e.className&&(this.table.className=e.className),this.initRenderer(e.render)},a.prototype.initRenderer=function(e){e=e||{},this.htmlRenderer=new o(e),this.htmlRenderer.addRenderer(function(e){if("object"==typeof e.content){var t=new a;for(var n in e.content)e.content.hasOwnProperty(n)&&t.addRow([n,e.content[n]]);return t.parse()}},2)},a.prototype.get=function(e,t){var n=this;return"undefined"!=typeof e&&(n=this.select("x","=",e)),"undefined"!=typeof t&&(n=n.select("y","=",t)),n.fetch()},a.prototype.addClass=function(e){if(!e)return;return e instanceof Array&&(e=e.join(" ")),this.forEach(function(t){n.window.addClass(t,e)}),this.auto_update&&this.parse(),this},a.prototype.removeClass=function(e){if(!e)return;var t;return e instanceof Array?t=function(e,t){for(var r=0;r<t.length;r++)n.window.removeClass(e,t[r])}:t=n.window.removeClass,this.forEach(function(n){t.call(this,n,e)}),this.auto_update&&this.parse(),this},a.prototype._addSpecial=function(e,t){if(!e)return;t=t||"header";if("object"!=typeof e)return{content:e,type:t};var n=[];for(var r=0;r<e.length;r++)n.push({content:e[r],type:t});return n},a.prototype.setHeader=function(e){this.header=this._addSpecial(e)},a.prototype.add2Header=function(e){this.header=this.header.concat(this._addSpecial(e))},a.prototype.setLeft=function(e){this.left=this._addSpecial(e,"left")},a.prototype.add2Left=function(e){this.left=this.left.concat(this._addSpecial(e,"left"))},a.prototype.setFooter=function(e){this.footer=this._addSpecial(e,"footer")},a._checkDim123=function(e){var t=a.H.slice(0);for(var n=0;n<e.length;n++)if(!i.removeElement(e[n],t))return!1;return!0},a.prototype.updatePointer=function(e,t){if(!e)return!1;if(!i.in_array(e,a.H))return a.log("Cannot update invalid pointer: "+e,"ERR"),!1;if(t>this.pointers[e])return this.pointers[e]=t,!0},a.prototype._add=function(e,t,n,r,i){if(!e)return!1;if(t){if(!a._checkDim123(t))return a.log("Invalid value for dimensions. Accepted only: x,y,z."),!1}else t=a.H;var s=function(e){var n={};n[t[0]]=u,n[t[1]]=l?r+l:r,n[t[2]]=c?i+c:i,n.content=e,this.insert(new f(n)),this.updatePointer(t[0],n[t[0]]),this.updatePointer(t[1],n[t[1]]),this.updatePointer(t[2],n[t[2]])};n=n||this.pointers[t[0]],r=r||this.pointers[t[1]]+1,i=i||this.pointers[t[2]],"object"!=typeof e&&(e=[e]);var o=null;for(var u=0;u<e.length;u++)if(e[u]instanceof Array){for(var l=0;l<e[u].length;l++)if(e[u][l]instanceof Array){for(var c=0;c<e[u][l].length;c++)s.call(this,e[u][l][c]);c=0}else s.call(this,e[u][l]);l=0}else s.call(this,e[u]);this.auto_update&&this.parse(!0)},a.prototype.add=function(e,t,n){if(!e)return;var r=e instanceof f?e:new f({x:t,y:n,content:e}),i=this.insert(r);return i&&(this.updatePointer("x",t),this.updatePointer("y",n)),i},a.prototype.addColumn=function(e,t,n){return e?this._add(e,a.V,t,n):!1},a.prototype.addRow=function(e,t,n){return e?this._add(e,a.H,t,n):!1},a.prototype.parse=function(){var e=function(e,t){if(!e)return;t=t||"td";var n=r.createElement(t),i=this.htmlRenderer.render(e);return n.appendChild(i),e.className&&(n.className=e.className),n};if(this.table)while(this.table.hasChildNodes())this.table.removeChild(this.table.firstChild);var t=this.table,n,i,s;if(this.header&&this.header.length>0){var o=r.createElement("thead");n=r.createElement("tr"),this.left&&this.left.length>0&&n.appendChild(r.createElement("th"));for(s=0;s<this.header.length;s++)n.appendChild(e.call(this,this.header[s],"th"));o.appendChild(n),s=0,t.appendChild(o)}if(this.length){var u=r.createElement("tbody");this.sort(["y","x"]);var a=-1,f=this.first(),l=f.x,c=0;for(s=0;s<this.db.length;s++){a!==this.db[s].y&&(n=r.createElement("tr"),u.appendChild(n),a=this.db[s].y,l=f.x-1,this.left&&this.left.length&&(i=r.createElement("td"),n.appendChild(e.call(this,this.left[c])),c++));if(this.db[s].x>l+1){var h=this.db[s].x-(l+1);for(var p=0;p<h;p++)i=r.createElement("td"),i.className=this.missing,n.appendChild(i)}n.appendChild(e.call(this,this.db[s])),l=this.db[s].x}t.appendChild(u)}if(this.footer&&this.footer.length>0){var d=r.createElement("tfoot");n=r.createElement("tr");for(s=0;s<this.header.length;s++)n.appendChild(e.call(this,this.footer[s]));d.appendChild(n),t.appendChild(d)}return t},a.prototype.resetPointers=function(e){e=e||{},this.pointers={x:e.pointerX||0,y:e.pointerY||0,z:e.pointerZ||0}},a.prototype.clear=function(e){s.prototype.clear.call(this,e)&&this.resetPointers()},f.prototype=new u,f.prototype.constructor=f}("undefined"!=typeof node?node.window||node:module.exports,"undefined"!=typeof window?window:module.parent.exports.window,"undefined"!=typeof node?node:module.parent.exports.node)